@@ -26,16 +26,17 @@ class Octuple(MIDITokenizer):
     :param nb_velocities: number of velocity bins
     :param additional_tokens: specifies additional tokens (time signature, tempo)
     :param sos_eos_tokens: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary
+    :param mask: will add a MASK token to the vocabulary (default: False)
     :param params: can be a path to the parameter (json encoded) file or a dictionary
     """
     def __init__(self, pitch_range: range = PITCH_RANGE, beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
                  nb_velocities: int = NB_VELOCITIES, additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-                 sos_eos_tokens: bool = False, params=None):
+                 sos_eos_tokens: bool = False, mask: bool = False, params=None):
         additional_tokens['Chord'] = False  # Incompatible additional token
         additional_tokens['Rest'] = False
         # used in place of positional encoding
         self.max_bar_embedding = 60  # this attribute might increase during encoding
-        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, params)
+        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, mask, params)
 
     def save_params(self, out_dir: Union[str, Path, PurePath]):
         """ Override the parent class method to include additional parameter drum pitch range
@@ -224,12 +225,20 @@ class Octuple(MIDITokenizer):
         ticks_per_sample = time_division // max(self.beat_res.values())
 
         if self.additional_tokens['Tempo']:
-            tempo_changes = [TempoChange(int(self.tokens_to_events(tokens[0])[6].value), 0)]
+            event = self.tokens_to_events(tokens[0])[6]
+            if event.type == 'Tempo':
+                tempo_changes = [TempoChange(int(event.value), 0)]
+            else:
+                tempo_changes = [TempoChange(TEMPO, 0)]
         else:  # default
             tempo_changes = [TempoChange(TEMPO, 0)]
 
         if self.additional_tokens['TimeSignature']:
-            time_sig = self._parse_token_time_signature(self.tokens_to_events(tokens[0])[-1].value)
+            event = self.tokens_to_events(tokens[0])[-1]
+            if event.type == 'TimeSig':
+                time_sig = self._parse_token_time_signature(event.value)
+            else:
+                time_sig = TIME_SIGNATURE  # default TS if the first token got an erroneous value
         else:  # default
             time_sig = TIME_SIGNATURE
         ticks_per_bar = time_division * time_sig[0]
@@ -258,13 +267,13 @@ class Octuple(MIDITokenizer):
             tracks[program].append(Note(vel, pitch, current_tick, current_tick + duration))
 
             # Tempo, adds a TempoChange if necessary
-            if self.additional_tokens['Tempo']:
+            if self.additional_tokens['Tempo'] and events[6].type == 'Tempo':
                 tempo = int(events[6].value)
                 if tempo != tempo_changes[-1].tempo:
                     tempo_changes.append(TempoChange(tempo, current_tick))
 
             # Time Signature, adds a TimeSignatureChange if necessary
-            if self.additional_tokens['TimeSignature']:
+            if self.additional_tokens['TimeSignature'] and events[-1].type == 'TimeSig':
                 time_sig = self._parse_token_time_signature(events[-1].value)
                 if time_sig != (time_sig_changes[-1].numerator, time_sig_changes[-1].denominator):
                     current_time_sig_tick += (current_bar - current_time_sig_bar) * ticks_per_bar
@@ -306,15 +315,18 @@ class Octuple(MIDITokenizer):
         """
         raise NotImplementedError('tokens_to_track not implemented for Octuple, use tokens_to_midi instead')
 
-    def _create_vocabulary(self, sos_eos_tokens: bool = False) -> Vocabulary:
+    def _create_vocabulary(self, sos_eos_tokens: bool = None) -> Vocabulary:
         """ Creates the Vocabulary object of the tokenizer.
         See the docstring of the Vocabulary class for more details about how to use it.
-        NOTE: token index 0 is often used as a padding index during training
+        NOTE: token index 0 is often used as a padding index during training TODO several vocabs
 
-        :param sos_eos_tokens: will include Start Of Sequence (SOS) and End Of Sequence (tokens)
+        :param sos_eos_tokens: DEPRECIATED, will include Start Of Sequence (SOS) and End Of Sequence (tokens)
         :return: the vocabulary object
         """
-        vocab = Vocabulary({'PAD_None': 0})
+        if sos_eos_tokens is not None:
+            print(f'\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
+                  f'_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
+        vocab = Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask)
 
         # PITCH
         vocab.add_event(f'Pitch_{i}' for i in self.pitch_range)
@@ -341,10 +353,6 @@ class Octuple(MIDITokenizer):
         # PROGRAM
         vocab.add_event(f'Program_{program}' for program in range(-1, 128))
 
-        # SOS & EOS
-        if sos_eos_tokens:
-            vocab.add_sos_eos()
-
         # BAR --- MUST BE LAST IN DIC AS THIS MIGHT BE INCREASED
         vocab.add_event('Bar_None')  # new bar token
         vocab.add_event(f'Bar_{i}' for i in range(self.max_bar_embedding))  # bar embeddings (positional encoding)
