@@ -27,18 +27,19 @@ class OctupleMono(MIDITokenizer):
     :param nb_velocities: number of velocity bins
     :param additional_tokens: specifies additional tokens (time signature, tempo)
     :param sos_eos_tokens: adds Start Of Sequence (SOS) and End Of Sequence (EOS) tokens to the vocabulary
+    :param mask: will add a MASK token to the vocabulary (default: False)
     :param params: can be a path to the parameter (json encoded) file or a dictionary
     """
 
     def __init__(self, pitch_range: range = PITCH_RANGE, beat_res: Dict[Tuple[int, int], int] = BEAT_RES,
                  nb_velocities: int = NB_VELOCITIES, additional_tokens: Dict[str, bool] = ADDITIONAL_TOKENS,
-                 sos_eos_tokens: bool = False, params=None):
+                 sos_eos_tokens: bool = False, mask: bool = False, params=None):
         additional_tokens['Chord'] = False  # Incompatible additional token
         additional_tokens['Rest'] = False
         additional_tokens['Program'] = False
         # used in place of positional encoding
         self.max_bar_embedding = 60  # this attribute might increase during encoding
-        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, params)
+        super().__init__(pitch_range, beat_res, nb_velocities, additional_tokens, sos_eos_tokens, mask, params)
 
     def save_params(self, out_dir: Union[str, Path, PurePath]):
         """ Override the parent class method to include additional parameter drum pitch range
@@ -177,15 +178,18 @@ class OctupleMono(MIDITokenizer):
 
         return instrument, tempo_changes
 
-    def _create_vocabulary(self, sos_eos_tokens: bool = False) -> Vocabulary:
+    def _create_vocabulary(self, sos_eos_tokens: bool = None) -> Vocabulary:
         """ Creates the Vocabulary object of the tokenizer.
         See the docstring of the Vocabulary class for more details about how to use it.
         NOTE: token index 0 is often used as a padding index during training
 
-        :param sos_eos_tokens: will include Start Of Sequence (SOS) and End Of Sequence (tokens)
+        :param sos_eos_tokens: DEPRECIATED, will include Start Of Sequence (SOS) and End Of Sequence (tokens)
         :return: the vocabulary object
         """
-        vocab = Vocabulary({'PAD_None': 0})
+        if sos_eos_tokens is not None:
+            print(f'\033[93msos_eos_tokens argument is depreciated and will be removed in a future update, '
+                  f'_create_vocabulary now uses self._sos_eos attribute set a class init \033[0m')
+        vocab = Vocabulary({'PAD_None': 0}, sos_eos=self._sos_eos, mask=self._mask)
 
         # PITCH
         vocab.add_event(f'Pitch_{i}' for i in self.pitch_range)
@@ -204,10 +208,6 @@ class OctupleMono(MIDITokenizer):
         if self.additional_tokens['Tempo']:
             vocab.add_event(f'Tempo_{i}' for i in self.tempos)
 
-        # SOS & EOS
-        if sos_eos_tokens:
-            vocab.add_sos_eos()
-
         # BAR --- MUST BE LAST IN DIC AS THIS MIGHT BE INCREASED
         vocab.add_event('Bar_None')  # new bar token
         vocab.add_event(f'Bar_{i}' for i in range(self.max_bar_embedding))  # bar embeddings (positional encoding)
